<?php
namespace Modules\EST\Services;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\ORM\DateTime;
use Teknisa\Libs\Util\NewCode;

use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class LancEstoqueService extends \Zeedhi\Framework\Controller\Simple {

    protected $connection;
    protected $entityManager;
    protected $newCode;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, NewCode $newCode)  {
        $this->connection = $entityManager->getConnection();
        $this->newCode = $newCode;
        $this->entityManager = $entityManager;
    }
    //constantes com campos que serão gravados na tabela de log
    const IMPORTA_NFENTRADA_LOG_FIELDS = array('SEQLOG', 'DSMETODO', 'IDACAO', 'IDTIPINTEGR', 'XMLLOG', 'OBSLOG', 'DSLOGINT', 'DTINCLUSAO', 'NRLANCTONF', 'CDFORNECED', 'NRAUTOFORN', 'CDFILMOVI' );
    const IMPORTA_NFENTRADA_DRILL_LOG_FIELDS = array('SEQLOG', 'DSMETODO', 'IDACAO', 'OBSLOG', 'DSLOGINT', 'DTINCLUSAO', 'CDPRODUTO', 'NRSEQITPROD', 'NRSEQUITEM');
    const GET_MAX_SEQLOG = "SELECT NVL(MAX(SEQLOG),0) AS MAX_SEQLOG FROM USR_LOG_WEBSERVICE";
    
    public function lancamentoEstoque($dataSet){
        try {
            try{
                if(isset($dataSet['CDEXFILIAL']))
                    $cdfilial = $this->buscaEXFilial($dataSet['CDEXFILIAL'])[0]['CDFILIAL'];
                else
                    $cdfilial = $dataSet['CDFILIAL'];
            }catch (\Exception $e) {
                throw new \Exception('Codigo CDEXFILIAL não existe');
            }

            $chave = 'NRLANCTOUSR'.$cdfilial;
            
            $nrlanctousr = $this->geraNovoCodigo(1, $chave);
            $dataSet['NRLANCTOUSR'] = $nrlanctousr;
            $dataSet['CDFILIAL'] = $cdfilial;
            
            $insertCabec = $this->insertCabecalhoLancamentoEstoque($dataSet);
            
            $insertItens = $this->insertItensLancamentoEstoque($dataSet);
            
            $retornoProc = $this->CallProcedureUSR_IMPORTA_MERCADORIA($cdfilial, $nrlanctousr);
            
            return $retornoProc;
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function insertCabecalhoLancamentoEstoque($dataSet){
        try {
            $sql = "INSERT INTO USR_ENTR_MERC(IDENTRSAID, CDCODOC, CDEXFILIAL, CDFILIAL, DSDESCRICAO, DTENTRADA, DTINCLUSAO, NRLANCTOUSR, NRINSJURFORN, CDSERIE, CDRETIRADA)
                    VALUES (:P_IDENTRSAID, :P_CDCODOC, :P_CDEXFILIAL, :P_CDFILIAL, :P_DSDESCRICAO, :P_DTENTRADA, :P_DTINCLUSAO, :P_NRLANCTOUSR, :P_NRINSJURFORN, :P_CDSERIE, :P_CDRETIRADA)";
    
            $params = array(
                'P_IDENTRSAID'  => $dataSet['IDENTRSAID'],
                'P_CDCODOC'     => isset($dataSet['NRNOTAFISC'])? $dataSet['NRNOTAFISC'] : $dataSet['NRDOCUMENT'],
                'P_CDEXFILIAL'  => $dataSet['CDEXFILIAL']?? null,
                'P_CDFILIAL'    => $dataSet['CDFILIAL'],
                'P_CDRETIRADA'  => $dataSet['CDRETIRADA']?? null,
                'P_DSDESCRICAO' => isset($dataSet['DSLANCESTQ'])? $dataSet['DSLANCESTQ'] : $dataSet['DSDESCRICAO'],
                'P_DTENTRADA'   => isset($dataSet['DTLANCESTQ'])? $dataSet['DTLANCESTQ'] : $dataSet['DTENTRSAID'],
                'P_DTINCLUSAO'  => date('d/m/Y'),//$dataSet['DTINCLUSAO'],
                'P_NRLANCTOUSR' => $dataSet['NRLANCTOUSR'],
                'P_NRINSJURFORN'=> $dataSet['NRINSJUR']?? null,
                'P_CDSERIE'     => $dataSet['CDSERIE']?? null
            );
            
            $this->entityManager->getConnection()->executeQuery($sql, $params);
            
            return false;
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function insertItensLancamentoEstoque($dataSet){
        try {
            
            $sql = "INSERT INTO USR_ITEM_ENTR_MERC(CDPRODINTE, VRQUANTIDADE, VRUNITARIO, CDEXFILIAL, CDFILIAL, NRLANCTOUSR, SGUNIDMED,
                                                    CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE, DTVALILOTE, DTFABRPROD)
                    VALUES (:P_CDPRODINTE, :P_VRQUANTIDADE, :P_VRUNITARIO, :P_CDEXFILIAL, :P_CDFILIAL, :P_NRLANCTOUSR, :P_SGUNIDMED,
                            :P_CDALMOXARIFE, :P_CDLOCALESTOQ, :P_NRLOTEESTQ, :P_NRSUBLOTE, :P_DTVALILOTE, :P_DTFABRPROD)";
            
            $arrDrill = ['ITENS', 'ITEN', 'PRODUTOS', 'PRODUTO', 'PRODUCTS', 'PRODUCT'];
            foreach($arrDrill as $detalhe){
                if(isset($dataSet[$detalhe])){
                   $itens = $dataSet[$detalhe];
                   break;
                }
            }
            
            foreach($itens as $item){
                
                $params = array(
                    'P_CDPRODINTE'      => isset($item['CDPRODUTO'])? $item['CDPRODUTO']: $item['CDPRODINTE'],
                    'P_VRQUANTIDADE'    => isset($item['QTTOTLANCTO'])? $item['QTTOTLANCTO'] : $item['QTPRODUTO'],
                    'P_VRUNITARIO'      => isset($item['VRUNILANCTO'])? $item['VRUNILANCTO'] : $item['VRUNITARIO'],
                    'P_CDEXFILIAL'      => $dataSet['CDEXFILIAL']?? null,
                    'P_CDFILIAL'        => $dataSet['CDFILIAL'],
                    'P_NRLANCTOUSR'     => $dataSet['NRLANCTOUSR'],
                    'P_SGUNIDMED'       => $item['SGUNIDMED']?? null,
                    'P_CDALMOXARIFE'    => $item['CDALMOXARIFE']?? null,
                    'P_CDLOCALESTOQ'    => $item['CDLOCALESTOQ']?? null,
                    'P_NRLOTEESTQ'      => $item['NRLOTEESTQ']?? null,
                    'P_NRSUBLOTE'       => $item['NRSUBLOTE']?? null,
                    'P_DTVALILOTE'       => $item['DTVALILOTE']?? null,
                    'P_DTFABRPROD'       => $item['DTFABRPROD']?? null
                );
                $this->entityManager->getConnection()->executeQuery($sql, $params);
            }
            
            return false;
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function CallProcedureUSR_IMPORTA_MERCADORIA($cdfilial, $nrlanctousr){
        try{
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'USR_IMPORTA_MERCADORIA');
            $procedure->addParam(new Param('P_MENSAGEMERRO', Param::PARAM_OUTPUT, '', Param::PARAM_TYPE_STR, 3000));
            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCTOUSR', Param::PARAM_INPUT, $nrlanctousr, Param::PARAM_TYPE_STR));
            
            return $procedure->execute();
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function getMaxSeqLog() {
        try {
            $params = array();
            return $this->entityManager->getConnection()->fetchAll(self::GET_MAX_SEQLOG, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function gravaLogWebservice($xml, $metodo, $arrXml, $messageLog){
     /*   switch($metodo){
            case 'CadastraProduto': 
                $query = $this->montaQuerryLog(self::CADASTRO_PRODUTO_LOG_FIELDS, 'USR_LOG_WEBSERVICE');
                $params = $this->montaParamsLog(self::CADASTRO_PRODUTO_LOG_FIELDS, $arrXml, $xml, $metodo, 'RECEBER', $messageLog);
                $params['P_DSLOGINT'] = 'Erro ao cadastrar o Produto: '.$arrXml['CDPRODINT'].' - '.$arrXml['NMPRODUTO'].' Ativo: '.$arrXml['IDPRODATIVO']; //'mensagem custom';
                break;
            case 'CadastraFornecedor':
                $query = $this->montaQuerryLog(self::CADASTRO_FORNECEDOR_LOG_FIELDS, 'USR_LOG_WEBSERVICE');
                $params = $this->montaParamsLog(self::CADASTRO_FORNECEDOR_LOG_FIELDS, $arrXml, $xml, $metodo, 'RECEBER', $messageLog);
                $params['P_DSLOGINT'] = 'Erro ao cadastrar o Fornecedor: '.$arrXml['CGCFORNE'].' - '.$arrXml['NMFANTFORNE'].' Ativo: '.$arrXml['IDFORNATIV']; //'mensagem custom';
                break;
            case 'CadastraCliente' :
                $query = $this->montaQuerryLog(self::CADASTRO_CLIENTE_LOG_FIELDS, 'USR_LOG_WEBSERVICE');
                $params = $this->montaParamsLog(self::CADASTRO_CLIENTE_LOG_FIELDS, $arrXml, $xml, $metodo, 'RECEBER', $messageLog);
                $params['P_DSLOGINT'] = 'Erro ao cadastrar o Cliente: '.$arrXml['CGCCLIE'].' - '.$arrXml['NMFANTCLIE'].' Ativo: '.$arrXml['IDCLIEATIV']; //'mensagem custom';
                break;
            case 'ImportaNFEntrada' :
                $query = $this->montaQuerryLog(self::IMPORTA_NFENTRADA_LOG_FIELDS, 'USR_LOG_WEBSERVICE');
                $params = $this->montaParamsLog(self::IMPORTA_NFENTRADA_LOG_FIELDS, $arrXml, $xml, $metodo, 'RECEBER', $messageLog);
                $params['P_DSLOGINT'] = 'Erro ao cadastrar a NF Entrada N°: '.$arrXml['NRNOTAFISCAL'].' Referente a AF : '.$arrXml['NRAUTOFORNE'].' Fornecedor :'.$arrXml['CDFORNECED']; //'mensagem custom';
                if($arrXml['PRODUTO'][0]){
                    foreach($arrXml['PRODUTO'] as $produto){
                        $queryDrill = $this->montaQuerryLog(self::IMPORTA_NFENTRADA_DRILL_LOG_FIELDS, 'USR_LOG_WEBSERVICE_DRILL');
                        $paramsDrill = $this->montaParamsLog(self::IMPORTA_NFENTRADA_DRILL_LOG_FIELDS, $produto, $xml, $metodo, 'RECEBER', $messageLog, $params);
                        try{
                            $this->entityManager->getConnection()->executeQuery($queryDrill, $paramsDrill);
                        }
                        catch (\Exception $e) {
                            return $e->getMessage();
                        }
                    }
                }
                else{
                    $queryDrill = $this->montaQuerryLog(self::IMPORTA_NFENTRADA_DRILL_LOG_FIELDS, 'USR_LOG_WEBSERVICE_DRILL');
                    $paramsDrill = $this->montaParamsLog(self::IMPORTA_NFENTRADA_DRILL_LOG_FIELDS, $arrXml['PRODUTO'], $xml, $metodo, 'RECEBER', $messageLog, $params);
                    try{
                        $this->entityManager->getConnection()->executeQuery($queryDrill, $paramsDrill);
                    }
                    catch (\Exception $e) {
                        return $e->getMessage();
                    }
                }
                break;
            case 'ExportaAF' :
                $query = $this->montaQuerryLog(self::EXPORTA_AF_LOG_FIELDS, 'USR_LOG_WEBSERVICE');
                $params = $this->montaParamsLog(self::EXPORTA_AF_LOG_FIELDS, $arrXml, $xml, $metodo, 'ENVIAR', $messageLog);
                $params['P_DSLOGINT'] = 'Erro ao exportar AF N°: '.$arrXml['NRAUTOFORNE'].' Fornecedor: '.$arrXml['CDFORNECED'].' Pagamento: '.$arrXml['CDCONDFORN'].' Codigo Exportação: '.$arrXml['CDCENTCUST']; //'mensagem custom';
                if($arrXml['_PCITENS']['TKPCITEM'][0]){
                    foreach($arrXml['_PCITENS']['TKPCITEM'] as $items){
                        $queryDrill = $this->montaQuerryLog(self::EXPORTA_AF_DRILL_LOG_FIELDS, 'USR_LOG_WEBSERVICE_DRILL');
                        $paramsDrill = $this->montaParamsLog(self::EXPORTA_AF_DRILL_LOG_FIELDS, $items, $xml, $metodo, 'ENVIAR', $messageLog, $params);
                        try{
                        $this->entityManager->getConnection()->executeQuery($queryDrill, $paramsDrill);
                        }
                        catch (\Exception $e) {
                            return $e->getMessage();
                        }
                    }
                }
                else{
                    $queryDrill = $this->montaQuerryLog(self::EXPORTA_AF_DRILL_LOG_FIELDS, 'USR_LOG_WEBSERVICE_DRILL');
                    $paramsDrill = $this->montaParamsLog(self::EXPORTA_AF_DRILL_LOG_FIELDS, $arrXml['_PCITENS']['TKPCITEM'], $xml, $metodo, 'ENVIAR', $messageLog, $params);
                    try{
                        $this->entityManager->getConnection()->executeQuery($queryDrill, $paramsDrill);
                    }
                    catch (\Exception $e) {
                        return $e->getMessage();
                    }
                }
                break;
            case 'AtualizaPedidoVenda':
                $query = $this->montaQuerryLog(self::ATUALIZA_PEDIDOVENDA_LOG_FIELDS, 'USR_LOG_WEBSERVICE');
                $params = $this->montaParamsLog(self::ATUALIZA_PEDIDOVENDA_LOG_FIELDS, $arrXml, $xml, $metodo, 'RECEBER', $messageLog);
                $params['P_DSLOGINT'] = 'Erro ao atualizar status da NotaFiscal numero: '.$arrXml['NRNOTAFISCAL'].' chave de acesso: '.$arrXml['NRACESSONFE']; //'mensagem custom';
                break;
            case 'DevolucaoDeCompra' :
                $query = $this->montaQuerryLog(self::IMPORTA_NFENTRADA_LOG_FIELDS, 'USR_LOG_WEBSERVICE');
                $params = $this->montaParamsLog(self::IMPORTA_NFENTRADA_LOG_FIELDS, $arrXml, $xml, $metodo, 'RECEBER', $messageLog);
                $params['P_DSLOGINT'] = 'Erro ao cadastrar a NF Devolucao N°: '.$arrXml['NRNOTAFISCAL'].' Referente a AF : '.$arrXml['NRAUTOFORNE'].' Fornecedor :'.$arrXml['CDFORNECED']; //'mensagem custom';
                if($arrXml['PRODUTO'][0]){
                    foreach($arrXml['PRODUTO'] as $produto){
                        $queryDrill = $this->montaQuerryLog(self::IMPORTA_NFENTRADA_DRILL_LOG_FIELDS, 'USR_LOG_WEBSERVICE_DRILL');
                        $paramsDrill = $this->montaParamsLog(self::IMPORTA_NFENTRADA_DRILL_LOG_FIELDS, $produto, $xml, $metodo, 'RECEBER', $messageLog, $params);
                        try{
                            $this->entityManager->getConnection()->executeQuery($queryDrill, $paramsDrill);
                        }
                        catch (\Exception $e) {
                            return $e->getMessage();
                        }
                    }
                }
                else{
                    $queryDrill = $this->montaQuerryLog(self::IMPORTA_NFENTRADA_DRILL_LOG_FIELDS, 'USR_LOG_WEBSERVICE_DRILL');
                    $paramsDrill = $this->montaParamsLog(self::IMPORTA_NFENTRADA_DRILL_LOG_FIELDS, $arrXml['PRODUTO'], $xml, $metodo, 'RECEBER', $messageLog, $params);
                    try{
                        $this->entityManager->getConnection()->executeQuery($queryDrill, $paramsDrill);
                    }
                    catch (\Exception $e) {
                        return $e->getMessage();
                    }
                }
                break;
        } //Fim do switch
        
        try{
            $this->entityManager->getConnection()->executeQuery($query, $params);
        }
        catch (\Exception $e) {
            return $e->getMessage();
        }
        */
    }
    
    public function montaParamsLog($arrayParams, $arrayValues, $xml = null, $metodo = null, $enviarReceber = null, $messageLog, $params){

        $params = $params? $params : array();
    
        
        $arrayValues['CDCLIENTE'] = $arrayValues['CDCLIENTE'] ? $arrayValues['CDCLIENTE'] : $arrayValues['CGCCLIE'];
        $arrayValues['CDFORNECED'] = $arrayValues['CDFORNECED'] ?  $arrayValues['CDFORNECED'] : $arrayValues['CGCFORNE'];
        $arrayValues['CDPRODUTO'] = $arrayValues['CDPRODUTO'] ? $arrayValues['CDPRODUTO'] : $arrayValues['CDPRODINTE'];
        $arrayValues['NRAUTOFORN'] = $arrayValues['NRAUTOFORN'] ? $arrayValues['NRAUTOFORN'] : $arrayValues['NRAUTOFORNE'];
        $enviarReceber ? $arrayValues['IDTIPINTEGR'] = $enviarReceber : null;
        $metodo ? $arrayValues['DSMETODO'] = $metodo : null;
        $xml? $arrayValues['XMLLOG'] = $xml : null;
        
        
        $seqLog = self::getMaxSeqLog();
        $seqMaxLog = $seqLog[0]['MAX_SEQLOG']+1;
        
        $arrayValues['SEQLOG'] = $seqMaxLog;
        
        if($arrayValues['IDACAO']){
            switch($arrayValues['IDACAO']){
                case 1:
                    $arrayValues['IDACAO'] =  'INCLUSAO';
                    break;
                case 2:
                    $arrayValues['IDACAO'] =  'ALTERACAO';
                    break;
                case 3:
                    $arrayValues['IDACAO'] =  'EXCLUSAO';
                    break;
            }
        }
        
            
        foreach($arrayParams as $key => $value) {
            $name = 'P_'.$value;
                
            $params[$name] = $params[$name]? $params[$name] : $arrayValues[$value];
            
        }
        
        $params['P_DTINCLUSAO'] = date('d/m/y');
        $params['P_OBSLOG'] = $messageLog;
        
        return $params;
    }
    
    private function montaQuerryLog($fields, $tabela){
		$query = '';
		$campos = '';
		$values = '';
		
		foreach ($fields as $key => $value) {
		        if($campos == ''){
		            $campos = $campos.$value;
		            $values = $values   .':P_'.$value;
		        }
		        else{
				    $campos = $campos.', '.$value;
		            $values = $values.', '.':P_'.$value;
		        }
		}
		
		$query = 'INSERT INTO '.$tabela.' ('.$campos.') VALUES ('.$values.')';
		
		return $query;
	}
    
    public function montaParamsSQL($arrayParams) {
        $params = array();
        $dsCount = 0;    
        foreach($arrayParams as $name => $value) {
            $name = 'P_'.$name;
            if($name == 'P_DSNIVEL') {
                foreach($value as $rowNum  => $rowVal){
                    $params[$name.strval($rowNum+1)] = $rowVal;
                }
            } else {
                if($value == '?')
                    $value = null;
                    
                $params[$name] = $value;
            }
            
        }
        
        $params['P_DTGRAVACAO'] = date('d/m/y');
        $params['P_HRGRAVACAO'] = date('H:i');

        return $params;
    }

	public function xml2array($xmlObject, $out = array ()){
        foreach ( (array) $xmlObject as $index => $node )
            $out[$index] = ( is_object ( $node ) ) ? $this->xml2array ( $node ) : $node;
    
        return $out;
    }
    
    public function obj_to_array($data){

            if (is_array($data) || is_object($data)){
                $result = array();
                foreach ($data as $key => $value){
                    $result[$key] = $this->obj_to_array($value);
                }
                return $result;
            }
            return $data;
    }
    
    public function buscaEXFilial($cdexfilial){
        try {
            $sql = "SELECT CDFILIAL
                    FROM PARAMFILIAL
                    WHERE CDEXFILIAL = :P_CDEXFILIAL";
    
            $params = array(
                'P_CDEXFILIAL'      => $cdexfilial
            );
            
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function geraNovoCodigo($nrorg, $cdcontador){
        try {
            /** @var Novocodigo $novocodigo */
            $novocodigo = $this->newCode->getCodeByTableName($cdcontador, $nrorg);
            return $novocodigo;
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }
    
}