IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
                    WHERE
	        O.NAME = 'ETL_VENDAFORSALE' AND
	        TYPE = 'P' 	            AND
	        U.UID = O.UID               AND
	        U.NAME = USER)
	DROP PROCEDURE ETL_VENDAFORSALE
GO

CREATE PROCEDURE ETL_VENDAFORSALE   @P_DTINI  DATETIME,
				    @P_DTFINA DATETIME
AS
   DECLARE @V_COUNTER       NUMERIC;
   DECLARE @V_DTFIM         DATETIME;
   DECLARE @V_DTINI         DATETIME;

   DECLARE @V_VRVENDAETLFOS DOUBLE PRECISION;
   DECLARE @V_VRIMPOSETLFOS DOUBLE PRECISION;
   DECLARE @V_QTITEMFETLFOS DOUBLE PRECISION;

   DECLARE @V_MES           VARCHAR(02);
   DECLARE @V_ANO           VARCHAR(04);
   DECLARE @V_BIM           VARCHAR(06);
   DECLARE @V_TRI           VARCHAR(06);
   DECLARE @V_SEM           VARCHAR(06);
   DECLARE @V_DSTIPOMOVFOS  VARCHAR(30);

   DECLARE @V_CDPRODUTO     VARCHAR(10); --PRODUTO.CDPRODUTO%TYPE;
   DECLARE @V_CDGRUPPROD    VARCHAR(10); --GRUPPROD.CDGRUPPROD%TYPE;
   DECLARE @V_IDTIPOMOVIVE  VARCHAR(1); -- MOVCAIXA.IDTIPOMOVIVE%TYPE;
   DECLARE @V_CDLOJA        VARCHAR(4);--LOJA.CDLOJA%TYPE;
   DECLARE @ACRESCIMO       DOUBLE PRECISION;
   DECLARE @DESCONTO        DOUBLE PRECISION;
   DECLARE @DTVENDA         DATETIME;
   DECLARE @DATAVENDA       VARCHAR(10);

   DECLARE @V_ZERO          NUMERIC;/*CONSTANTE*/

   DECLARE @V_CDFILIAL      VARCHAR(4);
   DECLARE @V_CDAREAGERE    VARCHAR(3);
   DECLARE @V_DTINICIALBI   DATETIME;

BEGIN
   SET @V_COUNTER = 0;
   SET @V_VRVENDAETLFOS = 0;
   SET @V_VRVENDAETLFOS = 0;
   SET @V_VRIMPOSETLFOS = 0;
   SET @V_ZERO = 0;

   SET @V_MES = (CONVERT (VARCHAR, MONTH(@P_DTINI)));
   SET @V_ANO = (CONVERT (VARCHAR, YEAR(@P_DTINI)));

   DELETE FROM ETL_FORSALE
   WHERE (CDANOBI > @V_ANO)
      OR (CDANOBI = @V_ANO AND CDMESBI >= @V_MES);
   -- COMMIT;

   SET @V_DTINI = CONVERT(DATETIME, ('01/'+@V_MES+'/'+@V_ANO), 103);

   WHILE (@V_DTINI <= @P_DTFINA) BEGIN
      SET @V_DTFIM = DATEADD(MM,1,@V_DTINI)-DAY(@V_DTINI);
	
      DECLARE C_ETL CURSOR FOR
         SELECT F.CDFILIAL, F.CDAREAGERE, F.DTINICIALBI
         FROM   FILIAL F
         WHERE  F.IDFUNCAOFILI IS NULL
           AND  F.DTINICIALBI IS NOT NULL;

      OPEN C_ETL;
      FETCH NEXT FROM C_ETL INTO @V_CDFILIAL,
                                 @V_CDAREAGERE,
				 @V_DTINICIALBI;

      WHILE (@@FETCH_STATUS = 0) BEGIN
         IF(   CONVERT(DATETIME,    ('01'+'/'+CONVERT(VARCHAR,MONTH(@V_DTINICIALBI))+'/'+CONVERT(VARCHAR,YEAR(@V_DTINICIALBI)))   ) <= @V_DTINI) BEGIN
            DECLARE C_VENDA CURSOR FOR
               SELECT 
                  IT.CDPRODUTO,   IT.CDGRUPPROD,   V.CDLOJA,
                  SUM(ROUND(IT.QTPRODVEND * (ISNULL(IT.VRUNITVEND,0)+ISNULL(IT.VRUNITVENDCL,0)),2,1) - ISNULL(IT.VRDESITVEND,0) + ISNULL(IT.VRACRITVEND,0))AS VRVENDAETLFOS ,
                  SUM(IT.QTPRODVEND) AS QTITEMFETLFOS,
                  SUM(IM.VRIMPOSETLFOS) AS VRIMPOSETLFOS,
                  SUM(IT.VRACRITVEND) AS ACRESCIMO,
                  SUM(IT.VRDESITVEND) AS DESCONTO,
                  DTVENDA
               FROM   
                  ITEMVENDA IT LEFT JOIN (SELECT CDFILIAL,CDCAIXA,NRSEQVENDA,NRSEQUITVEND,SUM(VRIMPOPRODIT) AS VRIMPOSETLFOS
                                          FROM ITVENDAIMPOS
                      		          GROUP BY CDFILIAL,CDCAIXA,NRSEQVENDA,NRSEQUITVEND) AS IM
                                          ON (IT.CDFILIAL     = IM.CDFILIAL    ) AND
                                             (IT.CDCAIXA      = IM.CDCAIXA     ) AND
                                             (IT.NRSEQVENDA   = IM.NRSEQVENDA  ) AND
                                             (IT.NRSEQUITVEND = IM.NRSEQUITVEND),
                  VENDA V
               WHERE     
                  V.CDFILIAL           =  @V_CDFILIAL
                  AND  V.DTENTRVENDA BETWEEN @V_DTINI AND @V_DTFIM
                  AND  V.IDSITUVENDA   <> 'C'      --Desconsidera as Vendas Canceladas
				      AND  V.CDFILIAL      =  IT.CDFILIAL
                  AND  V.CDCAIXA       =  IT.CDCAIXA                 
                  AND  V.NRSEQVENDA    =  IT.NRSEQVENDA
         	
               GROUP BY 
               	IT.CDPRODUTO, IT.CDGRUPPROD, V.CDLOJA, DTVENDA;
				
            OPEN C_VENDA;
            FETCH NEXT FROM C_VENDA INTO @V_CDPRODUTO, 
                                         @V_CDGRUPPROD, 
					 @V_CDLOJA,
					 @V_VRVENDAETLFOS, 
                  			 @V_QTITEMFETLFOS,  
					 @V_VRIMPOSETLFOS,
                                         @ACRESCIMO,
                                         @DESCONTO,
                                         @DTVENDA;

            WHILE (@@FETCH_STATUS = 0) BEGIN

               SET   @V_MES  =  CONVERT(VARCHAR, MONTH(@V_DTFIM));
               SET   @V_ANO  =  CONVERT(VARCHAR, YEAR(@V_DTFIM));
               SET   @V_BIM  =  CONVERT(VARCHAR, (ROUND(((@V_MES - 1) / 2),0) + 1))+' Bim';
               SET   @V_TRI  =  CONVERT(VARCHAR, (ROUND(((@V_MES - 1) / 3),0) + 1))+' Tri';
               SET   @V_SEM  =  CONVERT(VARCHAR, (ROUND(((@V_MES - 1) / 6),0) + 1))+' Sem';

               SET   @DATAVENDA =  CONVERT(VARCHAR, @DTVENDA, 103);

               INSERT INTO ETL_FORSALE
                  (
                  CDFILIAL,         CDAREAGERE,         CDFILILOJA,        
                  CDPRODUTO,        CDGRUPPROD,         CDANOBI,
                  CDMESBI,          BIETLFOS,           TRETLFOS,
                  SEETLFOS,         VRVENDAETLFOS,      VRIMPOSETLFOS,
                  QTITEMFETLFOS,    DTVENDAFOS,         VRTOTALACRES,
                  VRTOTALDESC
                  )
               VALUES
                  (
                  @V_CDFILIAL,      @V_CDAREAGERE,	@V_CDFILIAL+@V_CDLOJA,      
                  @V_CDPRODUTO,     @V_CDGRUPPROD,    	@V_ANO,
                  @V_MES,           @V_BIM,           	@V_TRI,                
                  @V_SEM,           @V_VRVENDAETLFOS, 	@V_VRIMPOSETLFOS, 
                  @V_QTITEMFETLFOS, @DATAVENDA,         @ACRESCIMO,
                  @DESCONTO
                  );
            					
/*		SET @V_COUNTER = @V_COUNTER+1;
		IF (@V_COUNTER >= 500) BEGIN
		   COMMIT;
		   SET @V_COUNTER = 0;
		END;*/
		FETCH NEXT FROM C_VENDA INTO @V_CDPRODUTO, 
		                             @V_CDGRUPPROD, 
                                             @V_CDLOJA,
                                             @V_VRVENDAETLFOS, 
            	      			     @V_QTITEMFETLFOS,  
					     @V_VRIMPOSETLFOS;
            END; --FIM DO WHILE PARA O SEGUNDO CURSOR: C_VENDA
	    CLOSE C_VENDA;
	    DEALLOCATE C_VENDA;
            END; --FIM DA CONDIO DE DATA PARA BI
            FETCH NEXT FROM C_ETL INTO @V_CDFILIAL,
                                       @V_CDAREAGERE,
                                       @V_DTINICIALBI;
         END; --FIM DO WHILE PARA O PRIMEIRO CURSOR: C_ETL
         CLOSE C_ETL;
         DEALLOCATE C_ETL;
         -- COMMIT;
         SET @V_DTINI = DATEADD(MM, 1, @V_DTINI);
      END; --FIM DO WHILE PARA VALIDAO DE PERIODO
END
GO

-- VERSAO --
EXEC ATUVEROBJ 'ELT_VENDAFORSALE','5.05.019';
GO


